/* $Id: cmd_info.c,v 1.5 2006/01/01 07:24:34 lace Exp $
 * client cmdline interface command "info" for libcaptive
 * Copyright (C) 2003 Jan Kratochvil <project-captive@jankratochvil.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; exactly version 2 of June 1991 is required
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "config.h"

#include <glib/gmessages.h>
#include <glib/gerror.h>
#include <popt.h>
#include <stdlib.h>
#include <string.h>
#include <captive/client-file.h>

#include "cmd_info.h"	/* self */
#include "cmd_cd.h"	/* for cmdline_path_from_cwd() */
#include "main.h"
#include "file_info.h"
#include "utf8.h"


GQuark cmdline_cmd_info_error_quark(void)
{
GQuark r=0;

	if (!r)
		r=g_quark_from_static_string("cmdline-cmd-info");

	return r;
}


const struct poptOption cmd_info_table[]={
		CMDLINE_POPT_AUTOHELP
		POPT_TABLEEND
		};


void cmd_info(const char **cmd_argv,GError **errp)
{
CaptiveFileObject *captive_file_object;
CaptiveFileInfoObject *captive_file_info_object;
const gchar *filepath;

	g_return_if_fail(!errp || !*errp);

	filepath=cmdline_path_from_cwd(cmd_argv[0]);

	if (!errvfsresult_to_gerr(errp,captive_file_new_open(
			&captive_file_object,	/* captive_file_object_return */
			cmdline_captive_vfs_object,	/* captive_vfs_object */
			filepath,	/* pathname */
			GNOME_VFS_OPEN_READ))) {	/* mode */
		err_cleanup(errp);
		g_set_error(errp,CMDLINE_CMD_INFO_ERROR,CMDLINE_CMD_INFO_ERROR_CANNOT_OPEN_FILE,
				_("Cannot open file: %s"),CMD_LOCALE_FROM_UTF8_ALLOCA(filepath));
		return;
		}

	if (!errvfsresult_to_gerr(errp,captive_file_file_info_get(
			captive_file_object,	/* captive_file_object */
			&captive_file_info_object))) {	/* file_info */
		err_cleanup(errp);
		g_set_error(errp,CMDLINE_CMD_INFO_ERROR,CMDLINE_CMD_INFO_ERROR_GETTING_FILE_INFO,
				_("Cannot get file information about: %s"),CMD_LOCALE_FROM_UTF8_ALLOCA(filepath));
		goto err_unref;
		}
	cmdline_captive_file_info_object_dump_full(captive_file_info_object,errp);
	g_object_unref(captive_file_info_object);
	if (*errp)
		goto err_unref;

err_unref:
	g_object_unref(captive_file_object);
}
